# Verilator-compatible testbench for the HPDcache block

This is a Verilator-compatible (with SystemC) testbench for the HPDcache block.

It is a standalone testbench where the HPDcache is not connected to a processor
core. Requests are generated by agents using a sequence defined by the user.
Some sequences are already provided in the sequence_lib subdirectory.

The testbench also implements a memory response model that acts as the main
memory of the system.

All the requests and responses to/from the HPDcache are validated automatically
by the testbench scoreboard.

## Prerequisites

This testbench requires the following tools and libraries pre-installed:

- [Verilator](https://verilator.org/guide/latest/install.html) - Version 5.018 or newer
- [SystemC Library](https://www.accellera.org/downloads/standards/systemc) - Version 2.3.4 or newer
- [SystemC Verification Library (SCV)](https://www.accellera.org/downloads/standards/systemc) - Version 2.0.1 or newer
- Perl - Version 5

Please follow the corresponding installation instructions for those packages to
install them in your system prior to the execution of this testbench.

You need to set the following environment variables (bash: `$ export`, csh: `$ setenv`):

- Set the SYSTEMC_LIBDIR to the libdir directory of the SystemC library installation
  (e.g. `$ export SYSTEMC_LIBDIR=<systemc-install-dir>/lib-linux64`)
- Add the bin/ subdirectory of the Verilator installation into your PATH
  (e.g. `$ export PATH=<verilator-install-dir>/bin:${PATH}`)

## Usage

Show the help message of the testbench:

```bash
$ make help
```

### The basic usage of the testbench is to execute the following sequence of commands

1. Verilate the RTL (convert to C++) and build the testbench:

Both stages with a single command:

```bash
$ make build -j<parallel_jobs>
```

Or both stages separately:

```bash
$ make verilate
$ make build -j<parallel_jobs>
```

If you have any errors or warnings, you can look into them in more detail on
their corresponding log files:

- build/verilate.log
- build/build.log

2. Run a simulation choosing one sequence. It can be for example the random
sequence (defined in the file sequence_lib/hpdcache_test_random_seq.h):

```bash
$ make run SEQUENCE=random LOG_LEVEL=1 NTRANSACTIONS=10000 SEED=42
```

If you need to debug the simulation waveforms, you can generate them in VCD
format using passing the TRACE=1 argument to the run command of the makefile:

```bash
$ make run SEQUENCE=random LOG_LEVEL=1 NTRANSACTIONS=10000 SEED=42 TRACE=1
```

Then to save some disk space, you can convert the VCD file to the FST format
using the vcd2fst script. For example, to convert the VCD file generated by the
previous command, you can use the following command:

```bash
$ ./scripts/vcd2fst.sh logs/run_random_42.vcd
```

This will replace the `logs/run_random_42.vcd` by the `logs/run_random_42.vcd.fst`.

### Non-regression suite

First build the testbench as explained above.

Then, the following example command will run 32 tests (with 32 different seeds)
using the random sequence:

```bash
$ make nonregression SEQUENCE=random LOG_LEVEL=1 NTRANSACTIONS=10000 NTESTS=32
```

### Logs

The build logs are written in the `build/` subdirectory. The simulation logs
are written in the `logs/` subdirectory.
